source procedure histogram( X@t, tau@t; hx@n) {
    language="java";
    maincode={{
      RegularGridData t = (RegularGridData) X.grid(0);
      RegularGridData n = (RegularGridData) hx.grid(0);
      double[] PDF = new double[n.ct()];
      for (int it=0; it<t.ct()-1; it++) {
         int inx;
         inx=(int)(X.realVal(it));
         if( inx>=0 && inx<=n.ct() ) 
         {PDF[inx]+=(tau.realVal(it+1)-tau.realVal(it));}
      }
      for (int in=0; in<n.ct(); in++) {
         double pdf = PDF[in];
        hx.set(in,pdf);
      }
    }};
}

      
// calculating data at regular grid spacing
source real function rg(tau@t,A@t,ts) {
  language="java";
   maincode={{
    RegularGridData t = (RegularGridData) tau.grid(0);
    double tsample = ts.realVal();
    int[] inx = new int[1];
    int[] inx2 = new int[1];
    double b=0.0;
    for (int i=0; i<t.ct()-1; i++) {     
      inx[0]=i;
      inx2[0]=i+1;
      if( (tsample>=tau.realVal(inx)) &&
          (tsample<=tau.realVal(inx2)) ) return A.realVal(inx);
    }
    return b;
  }};
}



//
import nsrunit;
unit conversion off;
math stochasticBrusselator {
/*
   STOCHASTIC MODELING
   
   Chemical kinetics with a finite number of molecules.

Consider the four reactions with the four rate constants
   A    -> X      k1        
   2X+Y -> 3X     k2
   B+X  -> Y + C  k3
   X    -> D      k4
where A, B, C, D are fixed and X and Y vary.
NOTE: to increase the number of molecules by a factor
of 10, divide k1 by 10, k2 by 1000, k3 by 100, and k4 by 10.
Be sure to make the corresponding change in the ODE model.

NOTE: It is not necessary to calculated C and D. 

Compute lambda1, lambda2, ... lambda4 as shown in the code
Sum the lambda's to calculate lambda.
Create a random exponential distribution by taking the natural
  log of uniform random numbers.
When multiplied by -1/lambda it becomes the time step when 
  another reaction takes place. The uneven time steps are
  accumulated in tau.
A second uniform random number for which reaction takes place
  is determined by the probability of the reaction which is the
  ratio of its lambda to the sum of all the other lambdas.
  We Calculate P1=lambda1/lambda, P2=(lambda1+lambda2)/lambda, ...
  For the reaction selected, the appropriate pools are 
  incremented or decremented in whole units.

When the run is finished, the data with uneven time increments
  may be regridded to even time increments. This is not an interpolation
  process. If tsample lies between tau(i) and tau(i+1), the
  corresponding value for A(tsample) is A(tau(i)).

  NOTE that the sampled data will have less variance and
  miss some of the excursions.
 
*/   
realDomain t sec; t.min=0; t.max=104; t.delta=0.1;
realState tau(t) sec;
when(t=t.min) {tau=0;}
realState X(t), Y(t);
real A=5, B=30, X0=30,Y0=30;
when(t=t.min) {X=X0; Y=Y0;}

real k1 = 0.1 sec^(-1);
real k2 = 0.001 sec^(-1);
real k3 = 0.01 sec^(-1);
real k4 = 0.1 sec^(-1);
real lambda1(t) sec^(-1);
real lambda2(t) sec^(-1);
real lambda3(t) sec^(-1);
real lambda4(t) sec^(-1);


lambda1=A*k1;
lambda2=X*(X-1)*Y*k2;
lambda3=B*X*k3;
lambda4=X*k4;
real lambda(t)=lambda1+lambda2+lambda3+lambda4;
real uni1(t) = random();
real uni2(t) = random();
real expDist(t);
expDist = -1.0/lambda*ln(uni1);
event(t>t.min) tau=tau+expDist;
real P1(t), P2(t), P3(t), P4(t);
P1=lambda1/lambda;
P2=(lambda1+lambda2)/lambda;
P3=(lambda1+lambda2+lambda3)/lambda;
P4=(lambda1+lambda2+lambda3+lambda4)/lambda;


event(uni2<P1) X=X+1;

event(uni2>=P1 and uni2<=P2) X=X+1;
event(uni2>=P1 and uni2<=P2) Y=Y-1;

event(uni2>P2 and uni2<=P3) X=X-1;
event(uni2>P2 and uni2<=P3) Y=Y+1;
event(uni2>P3) X=X-1;



// This section allows X and Y to be interpolated on a regularly spaced grid.
// FOR FASTER RUN TIME, COMMENT THIS SECTION OUT
/*
realDomain ts sec; ts.min=0; ts.max=tau(t.max)-t.delta; ts.delta=0.1;
real Xts(ts), Yts(ts);
Xts=rg(tau@t,X@t,ts);
Yts=rg(tau@t,Y@t,ts);
*/
// END OF SECTION TO COMMENT OUT

// Calculate histograms of time spent at various values of X and Y

realDomain n; n.min=0; n.max=150; n.delta=1;
real hx(n), hy(n);
histogram( X@t,tau@t,hx@n); 
histogram( Y@t,tau@t,hy@n);
real sumhx(n), sumhy(n);    // Cumulative Distributions
when(n=n.min) {sumhx=0;
               sumhy=0; }
sumhx:n = hx/tau(t.max);
sumhy:n = hy/tau(t.max);

}
