% Solves PRC-coupled system \dot(tht_i)=om_i + eps* PRC(tht_i) [1/N*Sum_j alpha_ij g(tht_j) + k_i I(t)] 
% Specify N, om, coupling matrix alpha_ij in parameters
% Specify functions PRC and g in subfunctions below
% I(t) is gsn. forcing ... into one or several cells -- SINGLE 'SHARED'
% process


clear all


%plot coupling functions
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    thtlist=linspace(0,1,1000);
    figure
    subplot(211)
    plot(thtlist,PRC2(thtlist)) ; ylabel('PRC z(\theta)')
    subplot(212)
    plot(thtlist,g2(thtlist)) ; ylabel('coup. fn. g(\theta)')


    
    
%model setup
tmax=10 %1000 ;    
dt=.001; %.00005 ; 


N=2;               %number of oscillators per system
om_onesystem=[1 1.1]';           %om, "baseline" phase velocity  1 gives period 1


%define OU forcing
tau_OU = .025;     %time const of OU process = tau_OU/la.
sig=1/sqrt(tau_OU) ; la=1 ;  %OU process:  equilib. distn is mean 0 gsn,
                 %std. dev. sig/sqrt(2*la)
		         %this choice limits to white noise w/rms1 as tau_OU --> 0:  std.
		         %dev = 1/sqrt(2 tau_OU)


coup_strength=1;

%enter matrix of coupling coefficients
coup_matrix_onesystem= coup_strength*[0 0 ; 1 0] ;  %GIVES FEED FWD COUPLING
                %coup_strength*[0 1 ; 1 0] ; % FEEDFWD AND FEEDBACK              
                %coup_strength*[0 1 ; 1 0] ;  % GIVES FEEDBACK COUPLING   
                                
                %coup_strength/(N-1))*(ones(N) - eye(N))  gives all to all coupling without self-coupling                      
k_onesystem=[1 0]' ;      %value 1 means gets OU input x, zero otherwise
                                                 
                                                 
num_init=500 %2500;   %number of different initial conditions that want to simultaneosly evolve


    %define coupling matrix for the entire system -- with repeating blocks
    %of coup_matrix_onesystem down main diagonal
 
coup_matrix = zeros(N*num_init);

for j=1:num_init
    coup_matrix(1+(j-1)*N:j*N,1+(j-1)*N:j*N)=coup_matrix_onesystem ; 
end

k=[]; om=[];
for j=1:num_init
    k=[k ; k_onesystem]; om=[om ; om_onesystem]; 
end

coup_matrix=sparse(coup_matrix);  %print to screen before sparsifying to check



    
    %recording data
    %TRAJECTORIES, PAIR OF CELLS
    tint=.1 ;      %temporal spacing between data points recorded FOR TRAJ OF PAIR OF SYSTEMS, make sure is multiple of dt
    tspan=linspace(0,tmax,round(tmax/tint));   
    tht_matrix=zeros(2*N,length(tspan)) ;   %record values of tht(t) for TWO of the systems
    recindex=1;
    
    
%SNAPSHOTS OF ALL OSCILLATORS
num_snap=10;
tint_snap=round(tmax/num_snap);  %temporal spacing between snapshots of all N*num_init points; make sure is mult. of dt
tspan_snap=linspace(0,tmax,round(tmax/tint_snap));   
    tht_matrix_snap=zeros(N*num_init,length(tspan_snap)) ;   %record values of tht(t) for ONE of the systems
    recindex_snap=1;




%initialize
    tht=1*rand(N*num_init,1)  ;    %initial conditions
    x=0;  %driving process
    t=0 ;


    
%%%%%%%%%%%%%%%%%%%%%%% EULER INTEGRATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%    
for n=1:ceil(tmax/dt)

    t=(n-1)*dt; 
    
    %report progress
    if mod(n,10000)==0
        fracdone=n/ceil(tmax/dt)
    end
   

    if mod(t,tint)==0
    tht_matrix(:,recindex)=tht(1:2*N);   %just data for pair of trajectories
    recindex=recindex+1;
    end

       
       
    if mod(t,tint_snap)==0
    tht_matrix_snap(:,recindex_snap)=tht;
    recindex_snap=recindex_snap+1;
    end

    
   x=x+ (-la*x*dt/tau_OU + sig*sqrt(dt)*randn(1)/sqrt(tau_OU)) ;   
   tht=tht + dt*( om + PRC2(tht).*(1/N*coup_matrix*g2(tht) + k*x) ) ;
   tht=mod(tht,1);
    
   
end  
%END   %%%%%%%%%%%%%%%%%%%%%%% EULER INTEGRATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%    



%plot snapshots of (tht1,tht2) for different initial conditions
figure
if N==2 ;    %else need to redo plotting, interpretation
    tht1_indices=linspace(1,num_init*N-1,num_init);
    tht2_indices=linspace(2,num_init*N,num_init);
 
    for j=1:num_snap    
    tht1list=tht_matrix_snap(tht1_indices,j) ;
    tht2list=tht_matrix_snap(tht2_indices,j) ;
    subplot(2,num_snap/2,j)
    plot(tht1list,tht2list,'.')   %these are matching coordinates for systems
    axis([0 1 0 1])
    axis square
    title(sprintf('t=%g',tspan_snap(j))); xlabel('\theta_1') ; ylabel('\theta_2')
    end
end




%plot trajectory (tht1,tht2) for single initial condition, vs time
figure
subplot(211)
plot(tspan,tht_matrix(1,:));  title('sample \theta_1(t)')
subplot(212)
plot(tspan,tht_matrix(2,:));  title('sample \theta_2(t)')





%plot distance vs. time bewteen traj with two diff initial conditions
figure
% get abs of difference between each of FIRST and SECOND coords,
% for the two diff traj
dist1= min( [mod(tht_matrix(1,:)-tht_matrix(3,:),1) ; mod(tht_matrix(3,:)-tht_matrix(1,:),1)] ) ;
dist2= min( [mod(tht_matrix(2,:)-tht_matrix(4,:),1) ; mod(tht_matrix(4,:)-tht_matrix(2,:),1)] ) ;
dist=sqrt(dist1.^2 + dist2.^2)
subplot(211)
plot(tspan,dist)
title('dist(t)')
subplot(212)
semilogy(tspan,dist)
title('dist(t)')


